@@ -96,7 +96,7 @@ pub fn decommit_stack_pages(addr: *mut u8, len: usize) -> Result<()> {
 /// the page fault handler will detect an out of bounds access and treat the page, temporarily,
 /// as a guard page.
 pub(super) fn initialize_memory_pool(pool: &MemoryPool) -> Result<()> {
-    if pool.memory_size == 0 || pool.max_wasm_pages == 0 {
+    if pool.memory_reservation_size == 0 || pool.max_memory_size == 0 {
         return Ok(());
     }
 
@@ -105,7 +105,7 @@ pub(super) fn initialize_memory_pool(pool: &MemoryPool) -> Result<()> {
             unsafe {
                 region::protect(
                     base as _,
-                    pool.max_wasm_pages as usize * WASM_PAGE_SIZE,
+                    pool.max_memory_size,
                     region::Protection::READ_WRITE,
                 )
                 .context("failed to initialize memory pool for uffd")?;
@@ -177,7 +177,7 @@ impl FaultLocator {
             max_instances: instances.max_instances,
             memories_start,
             memories_end,
-            memory_size: instances.memories.memory_size,
+            memory_size: instances.memories.memory_reservation_size,
             max_memories: instances.memories.max_memories,
         }
     }
@@ -262,13 +262,15 @@ unsafe fn initialize_wasm_page(
     page_index: usize,
 ) -> Result<()> {
     // Check for paged initialization and copy the page if present in the initialization data
-    if let MemoryInitialization::Paged { map, .. } = &instance.module.memory_initialization {
+    if let MemoryInitialization::Paged { map, .. } =
+        &instance.runtime_info.module().memory_initialization
+    {
         let memory_index = instance.module().memory_index(memory_index);
         let pages = &map[memory_index];
 
-        let pos = pages.binary_search_by_key(&(page_index as u64), |k| k.0);
+        let pos = pages.binary_search_by_key(&((page_index * WASM_PAGE_SIZE) as u64), |k| k.offset);
         if let Ok(i) = pos {
-            let data = instance.wasm_data(pages[i].1.clone());
+            let data = instance.wasm_data(pages[i].data.clone());
             debug_assert_eq!(data.len(), WASM_PAGE_SIZE);
 
             log::trace!(
@@ -436,30 +438,25 @@ impl Drop for PageFaultHandler {
 mod test {
     use super::*;
     use crate::{
-        Imports, InstanceAllocationRequest, InstanceLimits, ModuleLimits,
-        PoolingAllocationStrategy, Store, StorePtr, VMSharedSignatureIndex,
+        Imports, InstanceAllocationRequest, InstanceLimits, PoolingAllocationStrategy, Store,
+        StorePtr,
     };
     use std::sync::atomic::AtomicU64;
     use std::sync::Arc;
-    use wasmtime_environ::{Memory, MemoryPlan, MemoryStyle, Module, PrimaryMap, Tunables};
+    use wasmtime_environ::{Memory, MemoryPlan, MemoryStyle, Module, Tunables};
 
     #[cfg(target_pointer_width = "64")]
     #[test]
     fn test_address_locator() {
-        let module_limits = ModuleLimits {
-            imported_functions: 0,
-            imported_tables: 0,
-            imported_memories: 0,
-            imported_globals: 0,
-            types: 0,
-            functions: 0,
+        let instance_limits = InstanceLimits {
+            count: 3,
             tables: 0,
             memories: 2,
-            globals: 0,
             table_elements: 0,
             memory_pages: 2,
+            size: 1000,
+            ..Default::default()
         };
-        let instance_limits = InstanceLimits { count: 3 };
         let tunables = Tunables {
             static_memory_bound: 10,
             static_memory_offset_guard_size: 0,
@@ -469,7 +466,6 @@ mod test {
 
         let instances = InstancePool::new(
             PoolingAllocationStrategy::Random,
-            &module_limits,
             &instance_limits,
             &tunables,
         )
@@ -478,7 +474,7 @@ mod test {
         let locator = FaultLocator::new(&instances);
 
         assert_eq!(locator.instances_start, instances.mapping.as_ptr() as usize);
-        assert_eq!(locator.instance_size, 4096);
+        assert_eq!(locator.instance_size, 1008);
         assert_eq!(locator.max_instances, 3);
         assert_eq!(
             locator.memories_start,
@@ -497,7 +493,7 @@ mod test {
 
             let mut module = Module::new();
 
-            for _ in 0..module_limits.memories {
+            for _ in 0..instance_limits.memories {
                 module.memory_plans.push(MemoryPlan {
                     memory: Memory {
                         minimum: 2,
@@ -511,8 +507,6 @@ mod test {
                 });
             }
 
-            module_limits.validate(&module).expect("should validate");
-
             // An InstanceAllocationRequest with a module must also have
             // a non-null StorePtr. Here we mock just enough of a store
             // to satisfy this test.
@@ -573,28 +567,21 @@ mod test {
 
             let mut handles = Vec::new();
             let module = Arc::new(module);
-            let functions = &PrimaryMap::new();
 
             // Allocate the maximum number of instances with the maximum number of memories
             for _ in 0..instances.max_instances {
                 handles.push(
                     instances
                         .allocate(InstanceAllocationRequest {
-                            module: &module,
-                            memfds: None,
-                            unique_id: None,
-                            image_base: 0,
-                            functions,
+                            runtime_info: &super::super::test::empty_runtime_info(module.clone()),
                             imports: Imports {
                                 functions: &[],
                                 tables: &[],
                                 memories: &[],
                                 globals: &[],
                             },
-                            shared_signatures: VMSharedSignatureIndex::default().into(),
                             host_state: Box::new(()),
                             store: StorePtr::new(&mut mock_store),
-                            wasm_data: &[],
                         })
                         .expect("instance should allocate"),
                 );
