@@ -19,19 +19,24 @@
         clippy::use_self
     )
 )]
-#![cfg_attr(not(memfd), allow(unused_variables, unreachable_code))]
+#![cfg_attr(not(memory_init_cow), allow(unused_variables, unreachable_code))]
 
 use std::sync::atomic::AtomicU64;
+use std::sync::Arc;
 
 use anyhow::Error;
+use wasmtime_environ::DefinedFuncIndex;
+use wasmtime_environ::DefinedMemoryIndex;
+use wasmtime_environ::FunctionInfo;
+use wasmtime_environ::SignatureIndex;
 
 mod export;
 mod externref;
 mod imports;
 mod instance;
-mod jit_int;
 mod memory;
 mod mmap;
+mod mmap_vec;
 mod table;
 mod traphandlers;
 mod vmcontext;
@@ -39,6 +44,8 @@ mod vmcontext;
 pub mod debug_builtins;
 pub mod libcalls;
 
+pub use wasmtime_jit_debug::gdb_jit_int::GdbJitImageRegistration;
+
 pub use crate::export::*;
 pub use crate::externref::*;
 pub use crate::imports::Imports;
@@ -47,12 +54,10 @@ pub use crate::instance::{
     OnDemandInstanceAllocator, StorePtr,
 };
 #[cfg(feature = "pooling-allocator")]
-pub use crate::instance::{
-    InstanceLimits, ModuleLimits, PoolingAllocationStrategy, PoolingInstanceAllocator,
-};
-pub use crate::jit_int::GdbJitImageRegistration;
+pub use crate::instance::{InstanceLimits, PoolingAllocationStrategy, PoolingInstanceAllocator};
 pub use crate::memory::{DefaultMemoryCreator, Memory, RuntimeLinearMemory, RuntimeMemoryCreator};
 pub use crate::mmap::Mmap;
+pub use crate::mmap_vec::MmapVec;
 pub use crate::table::{Table, TableElement};
 pub use crate::traphandlers::{
     catch_traps, init_traps, raise_lib_trap, raise_user_trap, resume_panic, tls_eager_initialize,
@@ -67,15 +72,15 @@ pub use crate::vmcontext::{
 mod module_id;
 pub use module_id::{CompiledModuleId, CompiledModuleIdAllocator};
 
-#[cfg(memfd)]
-mod memfd;
-#[cfg(memfd)]
-pub use crate::memfd::{MemFdSlot, MemoryMemFd, ModuleMemFds};
+#[cfg(memory_init_cow)]
+mod cow;
+#[cfg(memory_init_cow)]
+pub use crate::cow::{MemoryImage, MemoryImageSlot, ModuleMemoryImages};
 
-#[cfg(not(memfd))]
-mod memfd_disabled;
-#[cfg(not(memfd))]
-pub use crate::memfd_disabled::{MemFdSlot, MemoryMemFd, ModuleMemFds};
+#[cfg(not(memory_init_cow))]
+mod cow_disabled;
+#[cfg(not(memory_init_cow))]
+pub use crate::cow_disabled::{MemoryImage, MemoryImageSlot, ModuleMemoryImages};
 
 /// Version number of this crate.
 pub const VERSION: &str = env!("CARGO_PKG_VERSION");
@@ -145,3 +150,48 @@ pub unsafe trait Store {
     /// completely semantically transparent. Returns the new deadline.
     fn new_epoch(&mut self) -> Result<u64, Error>;
 }
+
+/// Functionality required by this crate for a particular module. This
+/// is chiefly needed for lazy initialization of various bits of
+/// instance state.
+///
+/// When an instance is created, it holds an Arc<dyn ModuleRuntimeInfo>
+/// so that it can get to signatures, metadata on functions, memory and
+/// funcref-table images, etc. All of these things are ordinarily known
+/// by the higher-level layers of Wasmtime. Specifically, the main
+/// implementation of this trait is provided by
+/// `wasmtime::module::ModuleInner`.  Since the runtime crate sits at
+/// the bottom of the dependence DAG though, we don't know or care about
+/// that; we just need some implementor of this trait for each
+/// allocation request.
+pub trait ModuleRuntimeInfo: Send + Sync + 'static {
+    /// The underlying Module.
+    fn module(&self) -> &Arc<wasmtime_environ::Module>;
+
+    /// The signatures.
+    fn signature(&self, index: SignatureIndex) -> VMSharedSignatureIndex;
+
+    /// The base address of where JIT functions are located.
+    fn image_base(&self) -> usize;
+
+    /// Descriptors about each compiled function, such as the offset from
+    /// `image_base`.
+    fn function_info(&self, func_index: DefinedFuncIndex) -> &FunctionInfo;
+
+    /// Returns the `MemoryImage` structure used for copy-on-write
+    /// initialization of the memory, if it's applicable.
+    fn memory_image(&self, memory: DefinedMemoryIndex)
+        -> anyhow::Result<Option<&Arc<MemoryImage>>>;
+
+    /// A unique ID for this particular module. This can be used to
+    /// allow for fastpaths to optimize a "re-instantiate the same
+    /// module again" case.
+    fn unique_id(&self) -> Option<CompiledModuleId>;
+
+    /// A slice pointing to all data that is referenced by this instance.
+    fn wasm_data(&self) -> &[u8];
+
+    /// Returns an array, indexed by `SignatureIndex` of all
+    /// `VMSharedSignatureIndex` entries corresponding to the `SignatureIndex`.
+    fn signature_ids(&self) -> &[VMSharedSignatureIndex];
+}
